function [fBest,domBest,xBest,fMax]= DomByDom (x0, middle,triangle, ...
  structure,smooth,quantis)
%{
 Version: 2017-12-15

Here we know the domains, i.e. the basins and the plateaus.
Usually each basin is a local unimodal landscape (not always, though
 it depends on the value of "smooth".
In that case it is simple to find its minimum by an iterative method.
Here we apply a Matlab tool (fmincon)

 Outputs
  fBest = best solution (minimum value)
domBest = rank of the domain that contains the minimum
		xBest = best position
  
   
  %}
  clear valuex
 
  % Number of domains, and dimension
  [nbDom,D1]=size(triangle);
  D=D1-1;
  
options = optimoptions(@fmincon,...
    'Display','off'); %'Algorithm', 'interior-point');
  
  %---------------------------
  h = waitbar(0,'I am looking for the minimum. Please wait...'); %*****
  for nDom=1:nbDom % For each domain ...
    waitbar(nDom/nbDom)
    fun=@(xx)valuex(xx, x0,middle,triangle, structure,smooth,nDom);

Dtriangle=triangle(nDom,:);
LB=min(x0(Dtriangle,:)); UB=max(x0(Dtriangle,:));
%LB=zeros(1,D); UB=ones(1,D);

%{
% Remark:
The above constraints for the position x may be refined:
1) Method 1
For each hyperplan that is a boundary of the D-triangle
  - build its linear formula H
  - say H(x) and H(middle) must have the same sign

2) Method 2
Use barycentrics coordinates, and say they must be all >=0

However experiments tend to show that the result is not always better

%}

xStart = middle(nDom,:);
[xMin(nDom,:),fMin(nDom)]=fmincon(fun,xStart,[],[],[],[],LB,UB,[],options);
        
  end % for nDom=1:nbDom
  close (h) % Close the wait bar
;
  [fBest,domBest]=min(fMin);
xBest=xMin(domBest,:);
fprintf('\n fBest %f',fBest)
fprintf('\n on ' );disp(xBest)
  
  mess=[num2str(fBest) ' in the domain ' num2str(domBest)];
  h=msgbox(mess,'Probable minimum');
  h.Units='normalized';
  h.Position=[0.35,0.23 0.15 0.06];
  
  fprintf('\n Probable min %f in the domain %i, and on \n ',...
    fBest,domBest);disp(xBest)
 
  f=structure(1:nbDom,2:D+2);
  fMax=max(f(:));

  clear DA
end % End 
%==========================================================================      
%------------------------------------------------------------- initInDomain
function 	xNew=initInDomain(xTriangle,quantis)
[D1,D]=size(xTriangle);

%		Initialisation  (barycentric method)
% All variables are first supposed to be continuous
  w=rand(D1);
  w=w/sum(w);
  
  xNew(1:D)=w'*xTriangle; 


  % If some variable are discrete, it may be necessary to modify them
if  ~isempty(quantis)
		xNew(:)=quantify(xNew(:),quantis);
end

end
%----------------------------------------------------------------- quantify
function x=quantify(x,quantis)
	D=length(x);
	for d=1:D
		if quantis(d)>0
			x(d)=quantis(d)*round(x(d)/quantis(d));
		end
	end
end

%------------------------------------------------------------- keepInDomain
function x=keepInDomain(x0,xTriangle,quantis)
%	Keeps the position inside the normalised search space
% possibly temporarily expanded

% Finds the barycentric coordinates
W=baryCoord(x0,xTriangle);

% They should be all negative
% If necessary re-initialises the position at random
  if min(W)<0
    x=initInDomain(xTriangle,quantis);
  else
    x=x0;
  end

end

%------------------------------------------------------------------- around
function xAround=around(x,N,stepMax)
% Samples a position "around" a given one
	D=length(x);
  sigma=0.5/N; % Empirical
  sigma=sigma*stepMax;
  nSum=5; % 5
  
	for d=1:D 
 		c=bellLike(-0.5,1,nSum); g=sigma*c; % J.  Bell-like perturbation        
  	xAround (d) = min(1,max(0,x(d)+g));
	end
end
%----------------------------------------------------------------- bellLike
function rnd=bellLike(shift, nRnd, nSum)
%	Sample nRnd random numbers from a bell-like distribution
% The higher nSum, the more similar to the Normal distribution
% The support is [shift, 1+shift]
% The mean is shift+0.5

	 rnd=zeros(1,nRnd);  
    for k=1:nSum 
    	rnd=rnd+rand(1,nRnd);
    end
    rnd=shift+rnd/nSum;    
end
