function [delta0,delta1,deltaE, bestBasinSize]=diffDelta(x,middle,...
  triangle,structure,smooth,quantis)
% Estimate different difficulty measures for landscapes on D-triangles.
% The definition space is supposed to be normalised as [0,1]
delta0=0;
delta1=0;

delta0p=0;
coeffPlateau=1;

deltaE=0;
bestBasinSize=[];

[nbDom,D3]=size(structure);
D=D3-3;

beta=log(1/2)/log(4/5); %  So that difficulty=0.5 for median landscape

% ------------------------------------------------------------------Analyse
% Which basin contains the position of the minimum?

%if D<=2
    % For low dimension, this should be OK
 %   [miniGlobal,iMin]=min(structure(:,D3));
 %   fprintf('\n Probable min %f in domain %i',miniGlobal,iMin);
%else
    %{
      Remark: Actually it is just the minimum of the "middle points"
    Sometimes, because of the interpolation inside D-triangles,
    this is not the real minimum.
    So, we need a sub-optimisation inside each basin to be 
    more or less sure
    %}
    % Note: no quantis for the moment 
    
   % [miniGlobal,iMin,xBest]=greedy(x,middle,triangle,structure,smooth,quantis);
    [miniGlobal,iMin,xBest,fMax] =DomByDom (x, middle,triangle, ...
                              structure,smooth,quantis);
     
%end
fprintf('\n Domain of the global best: %i',iMin)
    if structure(iMin,1)==0
      fprintf('\n Note this is a plateau');
    end
    maxMini=max(structure(:,D3)); % ... similar remark

% ---------------------------------------------------------------Difficulty
% Domain sizes
for i=1:nbDom
   domainSize(i)=domainVolume(x,triangle(i,:)); 
end
domainSize=domainSize/sum(domainSize);

bestBasinSize=domainSize(iMin); % Size of the domain in which is
                                      % the solution

for i=1:nbDom 
  diff(i,1)=structure(i,1); %Code (plateau or basin)
  
  if i==iMin % To find the minimum ...
    diff(i,2)=1-domainSize(i); % ... the smaller the global basin, 
                       % the higher the difficulty to find it
    delta0=delta0+1/domainSize(i);
    delta1=delta1+1/domainSize(i);
     delta0p=delta0p+1/domainSize(i);
  else
    %The larger the local basin  or plateau
        % the easier to escape it
          diff(i,2)=domainSize(i);
          
        % If the local minimum value is near to the global one
        % it is more difficult to escape
        diff(i,2)=diffWeight(diff(i,2),structure(i,D3),miniGlobal,...
          maxMini);
        
        dDelta=(fMax-miniGlobal)*domainSize(i)/(structure(i,D+3)-miniGlobal);
    delta0=delta0+ dDelta;
    if structure(i,1)==0 % Plateau
      delta0p=delta0p+ coeffPlateau*dDelta;
   %fprintf('\nplateau, delta0p %f',delta0p);
     % If the local basin is far from the global one, 
    % it is more difficult to escape it
    distToMini=pdist2(xBest,middle(i,:));
    delta1=delta1+ (1+distToMini/sqrt(D))*coeffPlateau*dDelta; 
    else 
       delta0p=delta0p+ dDelta;
        %fprintf('\nbassin, delta0p %f',delta0p);
         distToMini=pdist2(xBest,middle(i,:));
    delta1=delta1+ (1+distToMini/sqrt(D))*dDelta; 
    end

   
  end
end

%--------------------------------- Sorts the list of partial difficulties
    [diff,Ind]=sortrows(diff, 2,'descend');
% fprintf('\n Difficulty of each domain:\n'); disp(diff(Ind,2))

% ----------------"Adds" the partial difficulties (relativity-like formula)
  deltaE=diff(1,2);
  for i=2:nbDom
    deltaE=(deltaE+ diff(nbDom,2))/(1+deltaE*diff(nbDom,2));
  end

% Applies beta to "normalise" the difficulty levels
% Just for easier comparisons, not really necessary
deltaE=deltaE^beta;

%-----------------  Map  to [0,1]
delta0=1-1/(1+delta0^0.5); %2*atan(delta0)/pi; %1-1/(1+delta0);
delta1=1-1/(1+delta1^0.5);
delta0p=1-1/(delta0p^0.5);

% The results in a box
h = msgbox(...
  sprintf('delta0: %f \ndelta0p: %f \ndelta1: %f  \ndeltaE: %f  \nBest domain size %f',...
  delta0,delta1,delta0p,deltaE,bestBasinSize),'Difficulty analysis');

htext = findobj(h, 'Type', 'Text');  %find text control in dialog
htext.FontSize = 15;     %set fontsize 
h.Units='normalized';
h.Position=[0.35,0.13 0.15 0.10];

%  The results in the command window 
fprintf('\n Difficulty delta0 %f \n',delta0)
fprintf('\n Difficulty delta0p %f \n',delta0p)

fprintf('\n Difficulty delta1 %f \n',delta1)
fprintf('\n Difficulty deltaE %f \n',deltaE)

fprintf('\n bestBasinSize %f \n', bestBasinSize)
end
%==========================================================================
function diffW=diffWeight(diff0,miniLocal,miniGlobal,maxMini);
  if abs(maxMini-miniGlobal)<2*eps % Unique plateau
    diff=diff0;
    return
  end
  
  deltaMini=(miniLocal-miniGlobal)/(maxMini-miniGlobal);
  
  % Linear
  diffW=diff0*(1-deltaMini);

end

