function [isInside,C]=inside(x,vert)
%{
 Checks whether x is inside the polyhedron or not
 defined by its D+1 vertices
 yes => isInside=true
 no => isInside=false
 method: computes the normalised barycentric weights (vector C)
     they must be all >=0. However we admit a certain tolerance
%}

D=length(x);
[nVert,Dv]=size(vert);
C=[];
tolerance=10^(-6);

if Dv~=D
  fprintf('\ninside. Error: dimension of x is %i',D);
  fprintf('\n   as the dimension of the vertices is %i',Dv);
  error(' ');
end

if nVert~=D+1
  fprintf('\ninside. The number of vertices is %i',nVert);
  fprintf('\n  It must be %i',D+1);
  error(' ')
end

switch D
  case 1
    isInside=(x-vert(1))*(x-vert(2))<=0;     
  
  otherwise
for v=1:nVert
  A(1:D,v)=vert(v,1:D)';
end

A(nVert,1:nVert)=ones(1,nVert);

deter=det(A); 
if abs(deter)<eps
  disp(A)
  fprintf('\n determinant %e',deter)
  error(' inside. Singular matrix ')
end

B=x';
%B(D+1)=1;
B(nVert)=1;

C=linsolve(A,B); % AC-B=0

isInside=true;

for c=1:nVert
  if C(c)<-tolerance % 
    isInside=false;
    break;
  end
end
end % switch D
end

