function [x,middle,triangle,structure,smooth]=landNew
x=[];
middle=[];
triangle=[];
structure=[];
smooth=1;

domainPlot=true; % Only for D=2
domainDisplay=false; % To display the positions defining the domains
middleChoice=0; % 0 => Gravity center (weights 1 for all vertices)
                % 1 => Barycenter (weights f(x_i) for the vertices
                % 2 => At random
%--------------------------------------------------------------------------
domain=[];

title = 'Defining the landscape';
prompt = {
  'Dimension',...
  'Number of vertices for the triangulation',...
  'Plateau "rate" (0=no plateau, 1=as many as possible")',...
  'All corners',...
  'Smoothness',...
  'Quick (but bad) triangulation',...
  'Deceptive, if possible'...
  };
defaultans ={ '1','5','0','y','1','N','n'};
options.Position=[200 800 250 300];
answer = inputdlg(prompt,title,1,defaultans,options);


if isempty(answer)
  D=0;
  return
end

N0=str2num(answer{2});
D=str2num(answer{1});
smooth=str2num(answer{5});

if D<=0
  errordlg('The dimension must be at least 1');
  D=0;
  return;
end

if N0==0
  % In that case the Delaunay triangulation does not work
  % (coplanar points). The QJ option joggles the inputs a bit.
  % See http://www.qhull.org/html/qdelaun.htm for more information
  triangOptions={'QJ'};
else
  triangOptions=[];
end

plateauRate=str2num(answer{3});

q=answer{4};
if q=='y' || q=='Y'
  corners=true; % Use all corners (2^D) of the search space
else
  corners=false;
end

q=answer{6};
if q=='y' || q=='Y'
  quickTriang=true;
else
  quickTriang=false;
end

q=answer{7};
if q=='y' || q=='Y'
  deceptive=true;
else
  deceptive=false;
end

%--------------------------------------------------------------------------
clear valuex % To clear the persistent variable DA in valuex.m

% The vertices of the domains
x=rand(N0,D); % At random inside the definition space
if corners 
 % Add the 2^D corners
 % If not, there will be just a "high plateau" around the convex envelop
 % defined by the N0 points
c=initCorners(zeros(1,D),0);
x=[x;c];
end
[N,~]=size(x);

if D==1 % Just in case polynom3 is true in value.m
  x=sort(x);
end

% The domains  (D-triangles)
triangle=triang(x,quickTriang,triangOptions);

[nbDom,D1]=size(triangle);

% The types  (plateau or basin)
Ind=randperm(nbDom);
nbPlateau=0;
nbMaxPlateau=round(plateauRate*nbDom);
structure(1:nbDom,1)=ones(1,nbDom)';

for nDom=1:nbDom
  if nbPlateau<nbMaxPlateau
    %ok=n==1 ||noCommon(triangle, Ind, n,structure);
    %if ok
    structure(Ind(nDom),1)=0;
    nbPlateau=nbPlateau+1;
    %end % if ok
  end
end % for n=1:nbDom

% The landscape  (i.e. the value on each vertex)
structure(1:nbDom,2:D+3)=nan(nbDom,D+2);

f=nan(1,N);
h = waitbar(0,'Generating the basins. Please wait...');

for nDom=1:nbDom
  if  structure(nDom,1)==0
    continue
  end
  waitbar(nDom/nbDom)
  for d=1:D+1
    node=triangle(nDom,d);
    if isnan(f(node)) % To ensure continuity
      r=rand;
      structure(nDom,d+1)=r;
      f(node)=r;
    else
      structure(nDom,d+1)=f(node);
    end
  end % for d=1:D+1
end
close(h)

h = waitbar(0,'Generating the plateaus. Please wait...');
% Note that two plateau can be side by side but wit different values
% (discontinuity)
for nDom=1:nbDom 
  waitbar(nDom/nbDom)
  if  structure(nDom,1)==1
    continue;
  end
      fPlateau=rand;
      f(triangle(nDom,:))=fPlateau;
    
    % Any point inside has the same value
    structure(nDom,2:D+3)=fPlateau;  
end
close(h)

% Now all vertices of this basin have a value
% For a point inside (the "middle"),
% assign a value lower than all the others

h = waitbar(0,'A value inside each basin. Please wait...');

for nDom=1:nbDom
   waitbar(nDom/nbDom)
 
  % Defines a point inside the domain
  v=x(triangle(nDom,:),:);
  
  if structure(nDom,1)==0 % Plateau
    middle(nDom,:)=mean(v,1); % Arbitray point (on the middle)
    
  else 
   % A value smaller than the ones of the vertices
    fMin=min(structure(nDom,1:D+2));
  r=0.5*rand*fMin;
  structure(nDom,D+3)=r;
  
  switch middleChoice
    
    case 1 % barycentric coordinates 
       for d=1:D+1
      aa(d)=fMin/structure(nDom,d+1);
    end
    aa=aa/sum(aa);
    middle(nDom,:)=aa*v; 
    
    case 2 % Random
     middle(nDom)=rand(1,D); 
    
    otherwise % Middle
    middle(nDom,:)=mean(v,1);
  end
   end
  
end % for nDom=1:nbDom
close(h)

 % Find the probable global min (not sure, though)
  [fMin,indMin]=min(structure(:,D+3));
  
 % Set it to zero
 structure(indMin,D+3)=0;
  
  %{
 % Shift in order to have f>=0, if possible
 if fMin<0
   for nDom=1:nbDom
     for d=2:D+3
       structure(nDom,d)=structure(nDom,d)-fMin;
     end
   end
 end
%}

if deceptive % Modify some values
  h = waitbar(0,'Trying to generate a deceptive landscape. Please wait...');

  % Position of the minimum
  xMin=middle(indMin,:);
  
  % For the other domain, set the min to a value decreasing
  % with the distance to xMin
  A=structure(:,2:D+2);
  fMax=max(A(:));

  for nDom=1:nbDom
       waitbar(nDom/nbDom)

    if nDom==indMin continue; end
    
    dd=pdist2(xMin, middle(nDom,:));
    fMiddle=structure(nDom,D+3);
    structure(nDom,D+3)=fMax*(1-dd/sqrt(D));
    fMiddle=structure(nDom,D+3);
    
    for d=2:D+2
      if structure(nDom,d)<fMiddle
        structure(nDom,d)=fMiddle+rand*(1-fMiddle); % At least a bit higher
      end
    end  
  end 
  close(h)
  
  % Check if really deceptive
  quantis=[]; % No discrete problem for the moment
  decept=NisB(x,middle,triangle,structure,smooth, quantis);
  
  if decept(1)>0.5 && decept(2)>0.5
       mess= msgbox('This landscape is almost surely deceptive');
  end
  
  if decept(1)>0.5 && decept(2)<=0.5
       mess= msgbox('Probably deceptive.',' ','warn');  
  end
  
   if decept(1)<=0.5 && decept(2)>0.5
       mess= msgbox('Might be deceptive.',' ','warn');  
   end

  if decept(1)<=0.5 && decept(2)<=0.5
    mess= msgbox('Not really deceptive, sorry.',' ','warn');
  end
    


    mess.Units='normalized';
    mess.Position=[0 0.72 0.15 0.09];
end

if domainDisplay
  domainDisp(x,triangle);
end


if D<=2
  landPlot(x,middle,triangle, structure,smooth);
else
  message={...
    'The landscape is now built, but can not be plotted' ...
    '(dimension > 2)' };
  h=msgbox(message);
end

% Ask for saving

fprintf('\n Dimension %i',D);
fprintf('\n Number of domains: %i',nbDom);
fprintf('\n Number of plateaus: %i \n',nbPlateau)
if quickTriang
  fprintf('\nWarning: as you use the quick triangulation')
  fprintf('\n the real number may be higher, because ')
  fprintf('\n the "inside" test can not work properly')
  endnb
  fprintf('\n Number of domains (D-triangles) %i\n',nbDom)
end

D=landSave(x,middle,triangle,structure,smooth);

end
%===========================================================================
function ok=noCommon(domainRank,Ind, n,structure)
%{
Cheks if the domain of rank Ind(n) as a common vertex  with
any domain of ranks Ind(1...n-1)
%}
ok=true;
for i=1:n-1
  c=intersect(domainRank(Ind(n),:),domainRank(Ind(i),:));
  if ~isempty(c) && structure(Ind(i),1)==0
    ok=false;
    break;
  end
end
end