function tri=triangQuick(pos)
%{
	
Builds an approximative triangulation.
(C) Maurice.Clerc@WriteMe.com  2015
Suggestions and bug reports are welcome

Last update: 2017-11
---------------------------------------------------------------------------
Quicker than Delaunay for relatively high dimensions
 but far to be perfect: the intersection of two D-triangles 
may be not empty. However, it is enough for some applications, 
like, for example, the generation of landscapes for optimisers 
(see LandGener)

This can certainly be improved.

pos			List of positions of dimension D
tri			List of "triangles" built on these positions.
					For D=1, a "triangle" is an interval
					For D=2, really a triangle
					For	D=3, a tetraedron
					etc.
%}
global Npopul % Population size. May be seful if triangQuick is called by
              %  an optimiser

[nbPos,D]=size(pos);

if exist('Npopul') && ~isempty(Npopul)
D=min(D,Npopul-1);  % In case of high D, we may have just a partial triangulation
						 % This is a compromise
end

switch D
  case 1 % The positions are supposed to be sorted

    for t=1:nbPos-1
      tri(t,1)=t; 
      tri(t,2)=t+1; 
    end
  %{  
  case 2
  	%pos=unique(pos,'rows');
    xp=pos(:,1);
    yp=pos(:,2);
    tri=delaunay(xp',yp');
   %} 
  otherwise
   
    %---First triangle
    % Finds the D nearest points not already used (if possible)
    Ind=knnsearch(pos(2:end,:),pos(1,:),'K',D);
    IndUsed=[1 Ind+1]; % True index in pos
    
    % Builds the triangle  
    for d=1:D+1
    	tri(1,d)=IndUsed(d);
    end
    
    % Updates the list of already used points
      posUsed=pos(IndUsed,:);
      
    % Updates the list of not yet used points
    posNotUsed=pos;
    IndUsed=sort(IndUsed,'descend');
    IndNotUsed=1:nbPos;

 		for n=1:length(IndUsed)
  		posNotUsed(IndUsed(n),:)=[];
  		IndNotUsed(IndUsed(n))=[];
  	end
  %fprintf('\n nbPos %i',nbPos)
 % fprintf('\n Used \n'); disp(IndUsed); 
 % fprintf('\n Not used \n');disp(IndNotUsed)	

   %---- Next triangles
    
  for n=2:nbPos-D
   
   % Finds the D already used points that are the nearest ones from
   % the first unused one
   Ind=knnsearch(posUsed(1:end,:),posNotUsed(1,:),'K',D);
  
   % Builds the triangle 
   	tri(n,1)=IndNotUsed(n-1);
    for d=2:D+1
    	tri(n,d)=IndUsed(Ind(d-1));
    end
    
   % Updates the list of already used points
   posUsed=[posUsed;posNotUsed(1,:)];
   
   % Updates the index of the already used points
   IndUsed=[IndUsed tri(n,1)];
 
 	% Updates the list of not yet used points
 		posNotUsed(1,:)=[];
 		
  
  end % for n=2:nbPos-D
    
end % switch D

end
