function f=valuex(xx, x,middle,triangle, structure,smooth,domainRank)
%{
Used for a function defined by a .txt file that contains
x, middle, triangle and structure (see LandGener)
OR
for a regular function (multiparaboloid) defined on D-squares
%}

global regular
global  nbParabD localMinF sizeA  sameMinF indParab 

persistent DA % Just to not to have to recompute it every time

if regular     
f=multiparaboloid(xx,nbParabD,localMinF, sizeA,  sameMinF,indParab); 
return;
end

[nbDom,D1]=size(triangle);
D=D1-1;

polynom3=false ; % General case: false

% Just for D=1. And it implies the "middle" position
%                 is not used, and replaced by (x_i-x_(i+1))/2
% Note; if you set here to false, it works, and with the real "middle"
% position, but the generated landscape is piece-wise linear

DD=D;

if isempty(DA)
  DA=nan(nbDom,D+2);
end

switch DD
  case 1 % In this case we use a polynomial of degree 3,
    % just for a nice look
    
    if ~exist('domainRank')
      % Finds the domain ins in which xx is
      isInside=false;
      for ins=1:nbDom
        xt=x(triangle(ins,:),:);
        x1=xt(1);
        x2=xt(2);
        
        if (xx-x1)*(xx-x2)<=0
          isInside=true;
          break;
        end
      end
      
      if ~isInside      
        [ind,dxx]=dsearchn(x,xx); % Nearest point, distance
        [tr,v]=find(triangle(:,:)==ind); % Which triangle and which vertex?
        f=structure(tr(1),1+v); % Value of the vertex
        f=f+dxx; % Slope
        return
      end
      
    else
    % if domainRank is given
      ins=domainRank;
      xt=x(triangle(ins,:),:);
      x1=xt(1);
      x2=xt(2);
    end
    
    if  structure(ins,1)==0 % Plateau
      f=structure(ins,2);
    else
      x3=(x1+x2)/2;
      f1=structure(ins,2);
      f2=structure(ins,3);
      f3=structure(ins,4);
    
      if polynom3
        if isnan(DA(ins,1))
          F=[f1, f2, f3, 0]';
          M=[
            x1^3 x1^2 x1 1;
            x2^3 x2^2 x2 1;
            x3^3 x3^2 x3 1;
            3*x3^2 2*x3 1 0
            ];
          
          DA(ins,1:4)=linsolve(M,F);
        end %   if isnan(DA(ins,1))
        
        f=DA(ins,1)*xx^3 + DA(ins,2)*xx^2 + DA(ins,3)*xx +DA(ins,4);
      
      else  % Two semi-parabolas
        x3m=middle(ins);
        if xx<=x3m
          coeff=(f1-f3)/(x1-x3m)^2;
          f=coeff*(xx-x3m)^2+f3;
        else
          coeff=(f2-f3)/(x2-x3m)^2;
          f=coeff*(xx-x3m)^2+f3;
        end
    
      end % if polynom3
    end % if plateau
    
  otherwise  % For D>1
    % Finds the domain ins in which x is
    if ~exist('domainRank')
      for ins=1:nbDom
        xt=x(triangle(ins,:),:);
        [isInside,~]=inside(xx,xt);
        if isInside
          break;
        end
      end
      
      if ~isInside
       [ind,dxx]=dsearchn(x,xx); % Nearest point, distance
        [tr,v]=find(triangle==ind(1)); % Which triangle and which vertex?
        f=structure(tr(1),1+v(1)); % Value of the vertex
        f=f+dxx; % Slope
        return
      end
      
    else % if domainRank is given
      ins=domainRank;
      xt=x(triangle(ins,:),:);
    end
    
    %At this point we know in which domain xx is   
    if structure(ins,1)==0 % If plateau, no need to compute anything
      f=structure(ins,2);
    else % For a basin, we build a nice unimodal local landscape
      
      % Builds and solves a linear system
      xt=[xt;middle(ins,:)]; % Add the "middle" point
      
      if isnan(DA(ins,1))
        F=structure(ins,2:D+3)';
        
        % Matrice M for interpolation
        M=zeros(D+2,D+2);
        
        % For each vertice, computes the distances
        % to all the others and to the middle
        for d1=1:D+1
          for d2=d1+1:D+2
            M(d1,d2)=norm(xt(d1,:)-xt(d2,:));
            M(d1,d2)=M(d1,d2)^smooth;
            M(d2,d1)=M(d1,d2);
          end
        end
        
        try
          DA(ins,:)=linsolve(M,F);
        catch
          fprintf('\nvalue. ins=%i',ins)
          fprintf('\n size of DA:'); disp(size(DA))
          fprintf('\n F'); disp(F)
          fprintf('\nM'); disp(M)
          %DA(ins,:)=linsolve(M,F);
        end
        
      end % if l==0 || l>ins
      
      % Distances from the position to the vertices and to the "middle"
      for d=1:D+2
        dd(d)=norm(xx-xt(d,:));
        dd(d)=dd(d)^smooth;
      end
   %disp(xx); disp(xt)
      %{
       Evaluates f as a linear combination of these distances.
       The coefficients are kept in DA for the next calls.
      They have been defined above to be sure the local lansdscape
      contains the D+1 vertices of the domain, and also its "middle"
      %}
      try
        f=DA(ins,:)*dd';
      catch
        fprintf('\nvalue. The position is inside the domain %i',ins);
        fprintf('\n There are %i domains ', nbDom);
        fprintf('\n but there is an error. Here is the size of DA:');
        disp(size(DA))
        error(' ')
      end
    end  %  if structure(i,1)==0
end
