function fit = func(x, fun0,~)
% Benchmark functions 
% INPUTS:
% -  x is a D-element row vector representing the candidate solution we want
%	 to evaluate, where D is the problem dimension
% -  fun0 is the index of the function of interest
% -  ~ is for opt_f, the minimum function value to try to reach. Not used here
%				but kept for consistency with some other benchmarks

% OUTPUT:
% -  fit is the fitness of x (i.e. fit = f(x))
global a % Just for tests
global x0 middle triangle  structure smooth  % Specific to functions built for difficulty analysis
global sunny
global xSol
global  nbParabD localMinF sizeA  sameMinF indParab % For Multi-paraboloid

	[~,D]=size(x);
	x=x';
  
    if fun0 >1000 && fun0 <2151 % Lennard-Jones
    	fun=5;
    else fun=fun0;
    end

switch fun
  case -6 % Biparaboloid

    %{
   persistent xSol1
   persistent xSol2
   persistent fmax
   persistent shift
    %}
   xSol1(1)=1/4;
   xSol2(1)=3/4;
   if D>1
   for d=2:D
     xSol1(d)=1/2;
     xSol2(d)=1/2;
   end
   end
   
   fmax=1/16+(D-1)/4;
   shift=0.002;
   
   xSol1=xSol1';
   xSol2=xSol2';
   dx1=x-xSol1;
   dx2=x-xSol2;

  dist1=norm(dx1);
	dist2=norm(dx2);
   
   if dist1<dist2
    f=dist1^2;
    fit=f/fmax;
   else
   f=dist2^2;
     f=f+shift;
     fit=shift+(1-shift)*f/fmax;
   end
 
  case -5 % Paraboloid [0,1]^D => [0,1], for calibration
    %						Solution on (0.5)*
		fit=0;
    mid=0.5;
		for d=1:D
			fit=fit+(x(d)-mid)^2;
		end
    fit=4*fit/D;
    
		case -4 % Needle (Parabola with plateau around)
		posMin=0.3*ones(1,D);
		width=0.001;
		r= pdist([x';posMin],'euclidean');
		if r<=width
			fit=r^2;
		else
			fit=width^2;
		end
	
		case -3 % Picece-wise linear
		if ~exist('a','var') a=0.3; end
		if x<a
			fit=(a-x)/a;
    else
			fit=(x-a)/(1-a);
		end

		case -2 % Line
		fit=1-x;
		%fit=x;
		
		case -1 % Impossible Plateau
		fit=1;

		case 0 % Paraboloid on [0,2D]*
%						Solution on (1, 2, 3, ..., D)
		fit=0;
		for d=1:D
			fit=fit+(x(d)-d)^2;
     % fit=fit+x(d)^2;
		end

    case 1 % Alpine. Minimum (1,2,...,D)
    	for d=1:D 
    		xd(d)=x(d)-d; 
        %xd(d)=x(d)-xSol(d);
      end
 
        fit = sum(abs(xd.*sin(xd)) + 0.1*abs(xd));
  
    case 2 % Tripod. Minimum (0, -50)		     			
    x1 = x(1) ; 
    x2 = x(2);
      
if x2<0
      fit=abs(x1)+abs(x2+50);
else
	 	if x1 < 0 
 			fit=1 + abs(x1+50) + abs(x2- 50);
		else 
			fit=2 + abs(x1-50) + abs(x2-50);
		end
end
     	case 3 % Gear train
     	z=1.0/6.931;
     	fit=z-x(1)*x(2)/(x(3)*x(4));
  		fit=fit*fit;
  
  		case 4 % Pressure vessel
  		fit=0.6224*x(1)*x(3)*x(4) + 1.7781*x(2)*x(3)*x(3) ...
  				+ x(1)*x(1)*(3.1661*x(4) +19.84*x(3));
  				
  		% Takes constraints into account (penalty method)
  		g(1)=0.0193*x(3)-x(1);
  		g(2)=0.00954*x(3)-x(2);
  		g(3)=1296000-pi*x(3)*x(3)*(x(4)+4*x(3)/3);
  	
  		pw=10^10;
  		if g(1)>0 c= 1+pw*g(1);  fit=fit*c*c; end
			if g(2)>0 c=1+pw*g(2); fit=fit*c*c;  end
			%if g(2)>0 c=1+g(2); fit=fit*c*c;  end
			if g(3)>0 c=1+pw*g(3); fit=fit*c*c;  end
			%if g(3)>0 c=1+g(3); fit=fit*c*c;  end
			
	
			case 5 % Lennard-Jones
			fit=lennardJones(x);
			
			case 6 % Flash
			c1=0.1;  
    c2=0.5;
    u1=x(1);
    if u1<2*c1 | u1>5*c1  fit=c2;
    else
      u2=u1-2*c1;
      if u2<=c1 fit=c2*(1-u2/c1);
      else
        u3=u1-3*c1;
        if u3<c1  fit=u3/c1;
        else
          u4=u1-4*c1;
          fit=1-c2*u4/c1;
        end
      end
    end
			
			case 7 % Comb
			c1=10; 
    c2=1;
    u=x(1);
    fit=1+sin(c1*u) +u/c1;
    fit=min(fit,c2);
			
			case 8 % Brush
			c1=10; c2=1;
    fit=0;
    for d=1:D
      xs=x(d)-d;
      fit=fit+abs(xs*sin(c1*xs))+abs(xs)/c1 ;
    end
    fit=min(fit,c2);
    
    case 9 % TSP6
    fit=TSP6eval(x); 
    
    case 91 % TSP
    fit=TSPeval(x); 
    
    case 10 % Canyon  on [0,10]
    border=9.99;
    if x<border
    	fit=0.8+x*0.2/border;
    else
    	fit=(10-x)/(10-border);
    end
    
    case 11 % Six Hump Camel Back
       x1=x(1); x2=x(2); 
       fit=	(4-2.1*x1^2+x1^4/3)*x1^2+x1*x2+(-4+4*x2^2)*x2^2;
       
    case 12 % Hartman 3D
    fit=hart3(x);
    
    case 13 % Hartman 6D
    fit=hart6(x); 
    
    
   case 14 % Griewank
   
   s=0; p=1;
   for d=1:D
   	s=s+x(d)^2;
   	p=p*cos(x(d)/sqrt(d));
   end
   fit=1+s/4000-p;
   
        
  case 15 % Sunnyspell, for D=2
    fit=sunnyspell(sunny,x);
    
  case 16 % Multi-paraboloid
    fit=multiparaboloid(x,nbParabD,localMinF, sizeA,  sameMinF, indParab);
    
  case 17 % Gas transmission compressor design problem
    x1=x(1); x2=x(2); x3=x(3);
    fit= (8.61*10^5) *sqrt(x1) *x2 *(x3^(-2/3)) *((x2^2-1)^(-1/2))...
      + (3.69*10^4) *x3...
      + (7.72*10^8) *(x2^0.219)/x1...
      -(765.43*10^6)/x1;
    %fit=abs(fit);
    
  case 18 % Three cubes
%{
 Looking for integers (a b c) so that Ntarget=a^3 + b^3 +c^3=(a,b,c)^3
Examples:
 216 = (3, 4, 5)^3 (and of course (6,-2, 2) because 216=6^3
 2001 = ( 10, 10, 1)^3
 2017 = (11, 7, 7)^3
 33 = (8 866 128 975 287 528, -8 778 405 442 862 239, -2 736 111 468 807 040)^3
%}

      Ntarget=33; %33; % Should be consistent with maxi in get_fun-info
      fit=x(1)^3 + x(2)^3 + x(3)^3;
      fit=abs(fit-Ntarget);
  
      case 10000 % Artificial functions (see Landscapes)
    fit=valuex(x',x0,middle, triangle,structure,smooth);

case 19 % For distance between hesitant Pythagorean fuzzy sets
    m1=x(1); m2=x(2);
 n1=x(3); n2=x(4);
 p1=x(5); p2=x(6);

    fit=abs(m1^2-n1^2) + abs(m2^2-n2^2);
    fit=fit+abs(n1^2-p1^2) + abs(n2^2-p2^2);
    fit=fit-abs(m1^2-p1^2) - abs(m2^2-p2^2);

    penalty=0;
    if m1^2+m2^2>1  penalty=10+penalty; end
    if n1^2+n2^2>1  penalty=10+penalty; end
    if p1^2+p2^2>1  penalty=10+penalty; end
      
    fit=fit+penalty;

    otherwise
        error('\nNo function %i',fun);
end



end

