function [LB, UB, D, opt_f, err,quantis] = get_fun_info(fun,D0)
% Get the lower bound, upper bound, dimension, and optimal value for function 'fun'
clear valuex
global net Atsp maxAtsp
global TSP cities
global sunny  % For Sunnyspell
global isOctave
TSP=false;

fprintf('\n get_fun_info fun %i',fun); 

LB=[];
UB=[];
quantis=[];
err=0.0001; % Default value
opt_f=0;

% See cases 9,91 ...
D=D0;

% Specific to functions built for difficulty analysis
global x0 middle triangle  structure smooth xStar fStar fMax
xStar=[]; % Solution point
fStar=[]; % Min value, if known
global xSol  % Solution point
xSol=[];
fMax=[]; % Max value, if known

switch fun
    
  case {-5 ,-6} % -5 =>Paraboloid [0,1]^D => [0,1], for calibration
                % -6 => Bi-paraboloid
     if exist('D0','var') && ~isempty(D0)
      D=D0;
    else D=1; % Default value
    end
    LB= zeros(1,D);
    UB= ones(1,D);
    err=0.001;
  
  case -4 % Needle (Parabola with plateau around)
    D=2;
    LB=zeros(1,D);
    UB=ones(1,D);
    
  case {-2, -3} % Line, Piece-wise linear
    D=1;
    LB=0;
    UB=1;
    
  case -1 % the Impossible Plateau
    if exist('D0','var') && ~isempty(D0)
      D=D0;
    else D=2; % Default value
    end
    
    LB=zeros(1,D);
    UB=ones(1,D);
    
  case 0 % Parabol, Paraboloid
    if exist('D0','var') && ~isempty(D0)
      D=D0;
    else D=1; % Default value
    end
    LB= zeros(1,D);
    UB= 2*D*ones(1,D);
    
  case 1 % Alpine. Minimum (1,2,...,D)
 
    if exist('D0','var') && ~isempty(D0)
      D=D0;
    else D=1; % Default value
    end
     xSol=1:D;
     fStar=0;
     %xSol=[9 2 5 4 3 8 1 7 6]; % Test. Update func.m accordingly)
     
    LB= zeros(1,D);
    UB= 4*D*ones(1,D);
    err =0;
  
    
  case 2 % Tripod. Minimum (0, -50)
    D=2;
    UB= 100*ones(1,D);
    LB= -UB;
    
  case 3 % Gear train
    % Min 2.700857e-12
    % (16 19 43 49)
    D=4;
    ones4=ones(1,4);
    LB=12*ones4;
    UB=60*ones4;
    quantis=ones4;
    % Actually, when looking at f (see func.m), it is easy to see that
    %	  for the minimum we must have
    %		x1, x2 in {13,14, ...,39}, and
    %		x3, x4 in {17, 18, ..., 60}
    
  case 4 % Pressure vessel
    % (constraints as penalties, see get_fun_info.m)
    % 100 runs, FEmax=4500
    % Theoretical solution:
    % f(0.8125 0.4375 42.0984455958549 176.6365958424394)=6059.714335048436 
    % "True global optimality of the pressure vessel design problem" by Xin-She Yang

    D=4;
    q=0.0625;
 
    LB=[q q 10 10];
    UB=[99 99 200 200];
 
    quantis=[q q 0 0];
    fStar=6059.714335048436;

  case 6 % Flash
    D=1;
    LB=0;
    UB=1;
    
  case 7 % Comb
    D=1;
    LB=0;
    UB=10;
    
  case 8 % Brush
    D=2;
    LB=[0 0];
    UB=[10 10];
 
  case {9,91} % TSP
  % Experimental. APS is not supposed to be good on this kind of problem.
    %global maxArow
    TSP=true;
     
    switch fun
    case 9
    %netData='TSP6_2.txt'; % (1 6 2 3 4 5) 20
    %netData='TSP4.txt'; % (4 3 1 2) 339
    netData='TSP6.txt'; % (1 2 5 4 6 3) 48
%netData='TSP5.txt'; % (1 2 3 5 4) 470
     
     case 91
    %-----------  TSPLIB burma14 %(10 9 11 8 13 7 12 6 5 4 3 14 2 1) 3323
    %{
        % Data computed by GeographicDistanceMatrixGenerator_v1.2.3
        netData='burma14distmeter.txt';
        net=dlmread(netData);
        net=round(net/1000);
    %}
    %-----------
    %{
        % Data computed by coordToDist.m, radius 6378.388
        netData='burma14distkm.txt'
        net=dlmread(netData);
        net=floor(net);
    %}
    %-----------
    
    % Data found on the Net
    % (http://esa.github.io/pygmo/tutorials/create_tsp.html)
    netData='burma14distkm0.txt' ;
    
    end
    
    net=dlmread(netData);
    
    %-----------
    [D, ~]=size(net);
    Atsp=net;  % Needed for plot
    maxAtsp=max(Atsp(:)); % Needed for penalties
    %{
        maxArow=0;
        for n=1:D
            maxArow=max(max(Atsp(n,:),maxArow));
        end
    %}
    %LB=ones(1,D);
    %UB=D*LB;
    % Better to define a slightly larger search space
    % Of course, only values in {1, ...,D} are finally acceptable
    % (see TSPeval)
    LB=zeros(1,D);
    UB=(D+1)*ones(1,D);
    
    % Plots the graph
    if isOctave
      % TO COMPLETE
     % G=drawGraph ( nodes, vertices); % No weight on the vertices
    else % Matlab
    G=graph(net);
    cities=plot(G,'EdgeLabel',G.Edges.Weight, 'NodeLabel',[]);
  
    axis off;
    cities.MarkerSize=30;
    for i=1:D
      text(cities.XData(i)-0.05,cities.YData(i),num2str(i),'fontsize',16);
    end
  end
  
    % Assigns a high cost to the non-existent arcs  
    for n1=1:D
      for n2=1:D
        if n1==n2
          continue
        else
          if net(n1,n2)<1
            net(n1,n2)=max(net(n1,:))*D +1;
          end
        end
      end
    end
    
  case 10 % Canyon
    D=1;
    LB=0;
    UB=10;
    
  case 11 % Six-hump Camel Back
    % min −1.031628453489877
    % on (±0.08984201368301331,±0.7126564032704135)
    D=2;
    LB=[-3,-2];
    UB=-LB;
    
  case 12 % Hartman 3D
    % min −3.86278214782076
    % on (0.1,  0.55592003,  0.85218259)
  D=3;
   LB=zeros(1,D);
    UB=ones(1,D);
    
     case 13 % Hartman 6D
  % min −3.32236801141551
  % on (0.20168952,0.15001069,0.47687398,0.27533243,0.31165162,0.65730054)  
  D=6;
   LB=zeros(1,D);
    UB=ones(1,D); 
    
    case 14 % Griewank 
    LB=-100*ones(1,D);
    UB=-LB;
    xStar=zeros(1,D); % Solution point
    
  case 15 % Sunnyspell
    %rng(123456789);
     D=2;
   LB=zeros(1,D);
    UB=ones(1,D); 
    sunny=rand(10,D);
    
  case 16 % Multi-paraboloid
    [LB,UB]=multiparaboloidDefine(D);
    %{
    global  nbParabD localMinF sizeA  sameMinF indParab 
    rng(123456789); % Initialise the RNG for reproducible landscape
                    % Modify it to have another landscape.
    %----------------------------------------------------------------------
    LB=zeros(1,D);
    UB=ones(1,D); 
    nbParabD=5; % Number of intervals on each dimension
    %             => nbParabD^D domains (D-cubes)
    sameMinF=0.5; % Common value of the local minima. 
    npd=nbParabD^D;
    localMinF=0.1 +0.9*rand(1,npd); % Values of the minima. 
    
    plateauRate=0;
    % Replace some paraboloids by a plateau
    nbPlateau=plateauRate*npd;
    for n=1:nbPlateau
      localMinF(randi(npd))=1;
    end
    
    % Define the global minimum position
    globMin=randi(npd);
    localMinF(globMin)=0;
    %globalMinRank=[1:D]-1; % Ranks of the global minimum (which paraboloid)
                            % along each dimension. Can be modified.
     sizeA=[]; for d=1:D sizeA=[sizeA nbParabD]; end;
   
     % Display its position                       
      [out{1:D}]=ind2sub(sizeA,globMin);
       indParab=cell2mat(out);
       fprintf('\n The theoretical minimum zero is on \n'); 
       disp((indParab-0.5)/nbParabD)
   
   %}
    
  case 17 % Gas transmission compressor design problem
    D=3;
    LB=[10 1.1 10];
    UB=[55 2 40];

  case 18 % Three cubes
    D=3;
    %maxi=2^53-1; 
    maxi=10^17; %1000; % To adapt to Ntarget. See func.m 
    UB=[maxi maxi maxi];
    LB=-UB;
    quantis=[1 1 1];

case 19 % For distance between hesitant Pythagorean fuzzy sets
    D=6;
    LB=zeros(1,D);
    UB=ones(1,D);


    
  case 10000  % Uses functions saved for difficulty analysis
    [x0,middle,triangle,structure,smooth,D,xSol,fStar,fMax]=landRead;
  fprintf('\n fStar %f, xStar', fStar); disp(xStar);
  err=0.0001;
  
    if D==0
      return
    end
    LB=zeros(1,D);
    UB=ones(1,D);
  
  otherwise
    if fun >1000 && fun <2151 % Lennard-Jones
      nbAtoms=fun-1000;
      % 	Best results in the literature (2015)
      % http://doye.chem.ox.ac.uk/jon/structures/LJ/tables.150.html
      % * means: found by APS with automatic FEmax, and at most 100 runs
      %		nbAtoms  Min
      %		3*				-3 (proved)
      %		4*				-6 (proved)
      %		5*		 		-9.103852415707552 (proved)
      %		6*		 		-12.712062
      % ...
      %		10*				-28.422532
      % ...
      % ...
      %		20*				-77.177043
      %		30*			-128.286571
      %		31			-133.586422
      
      D=3*nbAtoms;
      LB=-2*ones(1,D); UB=-LB;
      err=0;
      op_f=0;
      
    else
      error('function %i not implemented', fun)
    end
end

%if length(quantis)==0
 % quantis=zeros(1,D);
%end
end
