function [xBest, fBest,FEs, intermed,diffStop] = ...
  APS12 (funSet, fun, FEmax, opt_f,FuncAddr,LB,UB, expAlpha,...
  contractGamma,contractOption, ...
  restartRate,strategArray,N,verbose,FEintermed, keepRate, triosAll,...
  quantis,searchSpaceExpand, centerRate, density, rEquiv,expBeta,alphaRadius)

%{
 Adaptive Population-based Simplex. Portfolio version
 Proposed by: Maurice Clerc and Mahamed G.H. Omran

 Inputs
		See APS12_Monte

 Outputs
		xBest = position of the best agent
   fBest = best solution found (minimum value)
   FEs = number of FEs usable to solve the function "fun" (if possible)
		intermed= intermediate best values, for information and evolution curves

 Main differences from APS11
 The purpose was to more intensively search the _best_ solution,
  usually at the cost of a _worse_ median,
  particularly if the new parameter searchSpaceExpand
  is positive (see below)
  
 - local search is by sampling in a hypersphere. However it still an option
    (see around.m)
  
 - search space can slightly greater than the definition space. If a position
 leaves the definition space, then its value is nevertheless the one of
  the corresponding confined position
  
 - experimental TSP, seen as a quasi-continuous problem
  See MyFunctions/get_fun_info.m, case 91
  With plot of the graph, but no very good
  %}
  
  % Maximum number of strategies to use	during the search
  strategNb=length(strategArray);
  searchStat=zeros(2,10); % To save statistics about the search
  
  % If restart, number of agents to reinitialise
  Ninit=max(1,round(restartRate*N)); % At least 1
  
  % For difficulty analysis
  global xStar diffEps diffAnalysis
  %global fStary
  
  % For balance analysis
  global savePos 
  global xTot fTot
  global nExper nExploit exploType exploitK
  %global balanceE
  
  %------------------------------------------------------------------ Run  
  x=[];
  f=[];
  
  %		Initialises the positions
  %	Note they are sorted by increasing f
 
  FEs=0;
  [x,f,FEs]=initRand(N,LB,UB,x,f,fun,opt_f,FuncAddr,FEs,quantis,centerRate);
  diffStop=false; % Note that x is sorted by increasing f
  xTot=x;
  fTot=f;
  
  if savePos % For tests and balance analysis
    nExploit(nExper,1:2)=[0,0]; % The two first points are not exploitation
    
    for n=3:FEs % It may happen that a random initial point is inside
                % the exploitation domain of a previous one
     nExploit(nExper,n)=exploitCheck(xTot(1:n-1,:),xTot(n,:),...
                        nExploit(nExper,n-1),exploType,exploitK,alphaRadius);
     end
   %fprintf('\n ');disp(nExploit(nExper,:)) 
 
    end
  
  
  % 	Saves the current best result (value and position)
  fBest=f(1);
  xBest=LB+(UB-LB).*x(1,:);
  fprintf('\n Min after init: %1.4e',fBest);
  
  %		In case of automatic restarts, counts them for information
  restartNb=0;
  
  %------------------------------------------------------ Main loop -- 1.1
  intermed=Inf*ones(1,length(FEintermed));
  s=1; % Starts with the first strategy
  iter=0;
  
      %fprintf('\n size xTot');    disp(size(xTot)); fprintf(' / %i',length(fTot)) 

  
  while FEs<=FEmax && ~diffStop
    
    if verbose>0 fprintf('\n FE %i,  min %1.4e',FEs, f(1)); end
    iter=iter+1;
    % -------------------------- Successively tries the strategies -- 1.1.1
    fMinPrev=f(1); % Best value before moves (remember f is sorted)
    FEprev=FEs; % Number of FEs already used
    %fprintf('\n FEs before strategy %i: %i',s, FEs)
        
    [improv,FEs,searchStat,x,f]= ...
      strategApply(x,f, s,strategArray, expAlpha,contractGamma, ...
      contractOption,...
      fun, opt_f, LB, UB,FuncAddr, FEs, verbose, searchStat,...
      keepRate, triosAll,FEmax,quantis,searchSpaceExpand,...
      density, rEquiv,alphaRadius);
      

      % 	If partial restart method is used, tries to detect stagnation   
  coeff=1-keepRate; 
  Nnew=round(coeff*Ninit);
    Nnew=min(Nnew,FEmax-FEs);
Nnew=max(1,Nnew);
    %if Nnew>0 && restartRate>0
    if restartRate>0
      stagn=stagnation(fMinPrev,f(1),FEprev,FEs,expBeta);
  
      if stagn==1 % In case of probable stagnation, ...
        if FEs<=FEmax-Nnew % To be sure to no exceed the budget, even a bit
          restartNb=restartNb+1;
          % Random restart
         % fprintf('\n ** Reinit');
          FEsPrev=FEs;
          [x,f,FEs,xNew,fNew]= initRand(Nnew,LB,UB,x,f,fun,opt_f,FuncAddr, FEs,...
            quantis,centerRate);
          if savePos
           xTot=[xTot;xNew]; % Save all positions
           n1=length(fTot);
           n2=length(fNew);
           fTot=[fTot fNew];
            [~,nExLength]=size(nExploit);
            
            for m=1:n2 % It may happen that a random initial point is inside
                % the exploitation domain of a previous one
               n=nExLength+m;
   %disp(size(nExploit)); 
   %fprintf('\n %i nExLength',nExLength)
   
                nExploit(nExper,n)=exploitCheck(xTot(1:n1+m-1,:),xNew(m),...
                        nExploit(nExper,n-1),exploType,exploitK,alphaRadius);
            end 
          end  % if savePos   
        end
      end
    end

    
    if ~improv % If no improvement with the current strategy
      %           try the next one, cyclically
      if s==strategNb s=1; else s=s+1; end
    end
   
    
    % Saves intermediate results
    Ind=find(FEintermed<=FEs);
    indL=length(Ind);
    interm=max(1,indL);
    intermed(interm)=min(f);
    
    % Difficulty analysis (on function code (3, 10000))
    if  diffAnalysis==1 
      if ~isempty(xStar)
      % Warning: the search space is supposed to be normalised on [0 1]^D
      dx= pdist2(xStar,x(1,:));
      if  dx  <=diffEps
        diffStop=true;
      end
      else
       error('\nWidth   diffAnalysis==1  you must give xStar');    
      end
    end
    
    if  diffAnalysis==2 % && ~isempty(fStar)
      df=abs(fStar-f(1));
      if   df <=diffEps
        diffStop=true;
    %fprintf('\n f(1) %f fStar %f',f(1),fStar); error(' ')
        break;
      end
    end
  
   if diffStop break; end
   
  end %  while FEs<=FEmax  && ~diffStop ------------ End of main loop (1.1)
 

 
  % Saves the best result (value and position)
  fBest=f(1);
  xBest=LB+(UB-LB).*x(1,:);
  
  % Special cases
  if funSet==3 && fun==91 % TSP
    xBest=round(xBest); xBest=min(length(LB),max(1,xBest));
  end
  
  % Prints the result
  fprintf('\n  %i iterations',iter);
  fprintf('\n 	  %i	FEs =>	 %1.4e', FEs,fBest);
  
  if verbose>0
    fprintf('\nxBest');disp(xBest)
  end
  
  %		Prints final information
  fprintf('\nPopulation size %i',N);
  
  % 	Some stats
  fprintf('\nStrategy Nb Success Rate');
  for s=1:length(strategArray)
    str=strategArray(s);
    ss1=searchStat(1,s)
    fprintf('\n%i       %i   %i   %f',str, ss1,searchStat(2,s))
    if ss1>0 
      fprintf('    %f',searchStat(2,s)/ss1);
    else
      fprintf('    N/A')
    end
  end
  
  if restartRate>0
    fprintf('\n  Stagnations  %i',restartNb);
  end
  
end % End of APS





