function [improv, FEs,searchStat,x,f]=...
  strategApply(x,f,s,strategArray,expAlpha,contractGamma, ...
  contractOption,...
  fun, opt_f, LB,UB, FuncAddr,FEs,verbose, searchStat,...
  keepRate, triosAll,FEmax,quantis, searchSpaceExpand,...
  density, rEquiv,alphaRadius)
  
%{
   The strategy s has been selected, and it is
  now applied to sample new points.
  %}
  global isOctave
  global diffAnalysis diffEps
  global savePos xTot fTot
  global nExploit balanceE nExper exploType exploitK

  sortF=false; % To sort or not the three elements of a triplet
  contract=contractGamma;
  
  [N,D]=size(x);
  Ntemp=min(N,FEmax-FEs);
  fPrev=f; % *** ?
  fPrevInd=1:N;

 fTemp=[];
  toEval=ones(1,N); % A priori, all positions should be re-evaluated
  funcCount=0;
  
  % We will keep the eliteSize best positions for the next iteration
  keepSize=max(1,round(N*keepRate));
  if keepSize+Ntemp<N % Not enough budget for more search
    %											we stop here
    improv=0;
    FEs=FEmax+1;
    return;
  end
  
  strategNb=length(strategArray);
  strateg=strategArray(s);
  
  searchStat(1,s)=searchStat(1,s)+1; % For information.
  % Counts how many times each strategy is used
  
  % Modifies the probability threshold
  pv=probaV(x);
  
  switch strateg
    case 1  % Probabilistic expansion, dimension-wise
      trios=triplet(N,N,triosAll);
      
      for n=1:Ntemp
        r1=trios(n,1); r2=trios(n,2); r3=trios(n,3);
     
        if sortF
       % So that r1 is the best and r3 the worst
        f123=[f(r1) f(r2) f(r3)];
          [f123 Ind]=sort(f123);
          r1=Ind(1) ; r2=Ind(2); r3=Ind(3);
        end 
     
        for d=1:D
          if rand < pv % Semi-reflection 
            xTemp(n,d)=x(r3,d)+(1+expAlpha)*((x(r1,d)+x(r2,d))/2-x(r3,d));
          else
            xTemp(n,d)=x(n,d);
          end	%	if rand < pv
        end	%	 for d=1:D
        
        % It might happen that the position is not modified
        % In that case, we just "look around"
        if xTemp(n,:)==x(n,:)
          xTemp(n,:)=around(x,n,density, rEquiv);
        end
        
        % Checks confinement, and possibly adjusts the position;
       % if searchSpaceExpand>0 xTemp0(n,:)=xTemp(n,:); end
        xTemp(n,:) = handBound(xTemp(n,:),searchSpaceExpand);
        
      end	%	for n=1:Ntemp
  
  
    case 2  % Probabilistic contraction, dimension-wise
      trios=triplet(N,N,triosAll);
      
      for n=1:Ntemp
        r1=trios(n,1); r2=trios(n,2); r3=trios(n,3);
      
        if sortF
        % So that r1 is the best and r3 the worst
          f123=[f(r1) f(r2) f(r3)];
          [f123 Ind]=sort(f123);
          r1=Ind(1) ; r2=Ind(2); r3=Ind(3);   
        end
        
      switch contractOption 
        case 1 % Classical contraction
          for d=1:D
         if rand<pv
           %c=bellLike(0,0.5,2,3);
           c=0.5*rand(1,2); 
          xTemp(n,d)= x(r1,d)+( (c(1)*x(r2,d)+c(2)*x(r3,d))/2  - x(r1,d));
      
         else
            xTemp(n,d)=x(n,d);
          end
          end
         
        otherwise  % Pseudo-contraction
        for d=1:D
          if rand < pv
            if contractGamma<0 % To reduce bias. Note this is not always
              % really "contraction"
              contract=2*rand;
            end
            
            c=bellLike(0,1, 3,3); %Three random numbers from a bell-like distribution
           %c=bellLike(0,1, 3,5);
            c=c/sum(c); % => sum(c)=1
            xTemp(n,d)=(c(1)*x(r1,d)+c(2)*x(r2,d)+c(3)*x(r3,d))*contract; 
            %{
          Might be outside the search space, because "contract"
          On the other hand, with the true weighted gravity center,
         	results are not as good.
            %}
          else
            xTemp(n,d)=x(n,d);
          end % if rand < pv
        end % for d=1:D
           end % switch contractOption
      
        % It might happen that the position is not modified
        % In that case just look around
        if xTemp(n,:)==x(n,:)
          xTemp(n,:)=around(x,n,density, rEquiv);
        end
        
        % Checks confinement, and possibly adjusts the position;
        %if searchSpaceExpand>0 xTemp0(n,:)=xTemp(n,:); end
        xTemp(n,:) = handBound(xTemp(n,:),searchSpaceExpand);
       
      end % for n=1:Ntemp
      
    case {3 4}  % Local search. Tries to improve the best
      % It means that just one position is modified. The first one, in fact
      % for the list is sorted

       MaxFunEvals=FEmax-FEs;
        %MaxFunEvals=min(FEmax-FEs,3*N*D); % Empirical test
      %MaxFunEvals=min(FEmax-FEs,50); % Test
      
      ff = @(u)fEval(u,fun,opt_f,FuncAddr,FEs, LB, UB,quantis);
       xTemp=x;
     % if diffAnalysis>0
       % TolFun=diffEps; TolX=1.e-6; %TolX=diffEps;
     % else 
         TolFun=1.e-6; TolX=1.e-6;
     % end
      if isOctave         
       [xNew,fTemp(1),cvg,output] =nonlin_min(ff,x(1,:)');
       xTemp(1,:)=xNew';
       funcCount=output.nobjf;
        
      else  % Matlab   
      % fmincon.  (Matlab Optimization toolbox)
      %  http://fr.mathworks.com/help/optim/ug/constrained-nonlinear-optimization-algorithms.html    
      options = optimoptions(@fmincon,'MaxFunEvals',MaxFunEvals,...
        'Algorithm', 'interior-point',...
        'ObjectiveLimit',diffEps,...
        'Display','off','TolX',TolX, 'TolFun',TolFun);
      %  'Algorithm', 'interior-point',...
      %  'Algorithm', 'sqp', ...
      
      
      
      [xTemp(1,:),fTemp(1),~,output] = fmincon(ff,x(1,:),[],[],[],[],...
        LB,UB,[],options);
     % if searchSpaceExpand>0 xTemp0(1,:)=xTemp(1,:); end
      xTemp(1,:) = handBound(xTemp(1,:),searchSpaceExpand);
      funcCount=output.funcCount;
  
    end % if isOctave
     
     %{
     TO MODIFY
      FE=FEs+funcCount-FEmax;
       if FE>0 % Not enough budget for more search, we stop here
          improv=0;
          for fe=1:FE
            nExploit(nExper,FEs+fe)=nExploit(nExper,FEs+fe-1)+1 ;
          end

           FEs=FEmax+1
        return;
      end
      %}
      
     % toEval(1)=0; % xTemp(1) has been evaluated
     % The other positions have not been modified
      toEval(1:Ntemp)=zeros(1,Ntemp);
      
      if strateg==4 % Increases diversity
        % For other positions (and even 1 if the above local search did not
        % improve it, looks "around"
        
        if fTemp(1)>=f(1)
          xTemp(1,:)=around(x,1,density,rEquiv);
          toEval(1)=1;
        end
        
        k=1;
        N1=keepSize+1;
 
        for n=N1:N % Around "bad" positions
          k=k+1;
         % xTemp(k,:)=around(x,n,density4,r4);
         xTemp(k,:)=rand(1,D);
         % if searchSpaceExpand>0 xTemp0(n,:)=xTemp(k,:); end % Test
          xTemp(k,:) = handBound(xTemp(k,:),searchSpaceExpand);
        end
        Ntemp=k;
      end     
  end % switch strateg
  
  
  % Quantifies xTemp (if needed)
  if ~isempty(quantis)
    %x1=xTemp(1,:);
    for n=1:Ntemp
      x1=xTemp(n,:);
      xTemp(n,:)=quantify(xTemp(n,:),quantis);
     % if searchSpaceExpand>0 xTemp0(n,:)=xTemp(n,:); end
      xTemp(n,:) = handBound(xTemp(n,:),searchSpaceExpand);

    if xTemp(n,:) ~=x1
      toEval(1)=1; % Must be evaluated
    end
    end
  end
  
  %---------------Evaluates xTemp  (the new positions)

  if savePos && (strateg==3 || strateg==4)
    % We assume there has been funcCount exploitation points
    % for the local search
    % It may be wrong, though, depending on the method, and nExploit overvalued
    for n=1:funcCount
      nExploit(nExper,FEs+n)=nExploit(nExper,FEs+n-1)+1;
    end
  end
  
     FEs=FEs+funcCount;  
 
  if strateg==3 || strateg==4 % If local search only one position is modified
    Ntemp=1;
    end
  
  for n=1:Ntemp 
    if toEval(n)==1   && FEs<FEmax 
      
        [fTemp(n), FEs]=fEval(xTemp(n,:), fun, opt_f,FuncAddr,FEs, LB, UB,[]);
       
       if savePos
         % if strateg~=3 && strateg~=4
        nExploit(nExper,FEs)=exploitCheck(xTot,xTemp(n,:),...
        nExploit(nExper,FEs-1),exploType,exploitK,alphaRadius);
        %end
     
       xTot=[xTot;xTemp(n,:)];
       fTot=[fTot fTemp(n)];
       end   
    end
  end
  
  
  %	Adds the keepSize best ones
%  if searchSpaceExpand>0 % Test
 %   xAll=[x(1:keepSize,:);xTemp0];
%  else

    xAll=[x(1:keepSize,:);xTemp];
 % end
  fAll=[f(1:keepSize) fTemp];
  
  % To be sure to be able to keep at least N agents
  n=length(fAll);

  if n<N
    dN=N-n;
    xAll=[xAll;x(keepSize+1:keepSize+1+dN,:)];
    fAll=[fAll f(keepSize+1:keepSize+1+dN)];
  end
  
  % Sorts according to the fitness
  [fAll, Ind]=sort(fAll);
 %disp(size(xAll)); disp(max(Ind)) 
  
  xAll=xAll(Ind,:);
  
  % Keeps the N best ones for the next iteration
  %fPrev=f; %*** To check
 % fprintf('\n Ntemp %i N %i',Ntemp,N); disp(size(fAll))
  f=fAll(1:N);
  x(1:N,:)=xAll(1:N,:);
  
 fOld=[];
 for n=1:length(Ind)
   if Ind(n)<=N
     fOld=[fOld fPrev(Ind(n))];
   end
 end
 
  
  %	Estimates the number of improvements
  % Note that it is usually not the exact count, for we did not memorise
  % the ranks of the previous positions and f has been re-sorted
  improvNb=0;
  for n=1:N
   % if f(n)<fPrev(n)  % ??? ALWAYS false? (see fPrev)  2018-03-02
   if f(n)<fOld(n)
      improvNb=improvNb+1; 
    end
  end
  
  % Checks if "enough" improvement
  %if improvNb>keepSize  % A
  if rand<improvNb/N  % B
    improv=true;
    searchStat(2,s)=searchStat(2,s)+1;
  else
    improv=false;
  end

end % End of strategApply
